"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_container_gen_1 = require("ern-container-gen");
const lodash_1 = __importDefault(require("lodash"));
const semver_1 = __importDefault(require("semver"));
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const fs_1 = __importDefault(require("fs"));
const fast_levenshtein_1 = __importDefault(require("fast-levenshtein"));
const constants = __importStar(require("./constants"));
const treeify_1 = __importDefault(require("treeify"));
const got_1 = __importDefault(require("got"));
class Ensure {
    static isValidElectrodeNativeModuleName(name, extraErrorMessage = '') {
        if (!ern_core_1.utils.isValidElectrodeNativeModuleName(name)) {
            const errorMessage = `${name} is not a valid Electrode Native module name.\nCheck GLOSSARY for Electrode Native module naming rules.\nhttps://native.electrode.io/reference/glossary${extraErrorMessage}`;
            throw new Error(errorMessage);
        }
    }
    static isValidNpmPackageName(name, extraErrorMessage = '') {
        const validation = (0, validate_npm_package_name_1.default)(name);
        if (!validation.validForNewPackages) {
            const errorMessage = `${name} is not a valid npm package name\n`
                .concat(validation.errors ? validation.errors.join('\n') : '')
                .concat(`\n${extraErrorMessage}`);
            throw new Error(errorMessage);
        }
    }
    static async isNewerContainerVersion(descriptor, containerVersion, extraErrorMessage = '') {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const cauldronContainerVersion = await cauldron.getTopLevelContainerVersion(ern_core_1.utils.coerceToAppVersionDescriptor(descriptor));
        if (cauldronContainerVersion) {
            // If both versions are valid semver versions, use semver comparison
            // otherwise just use string lexographical comparison
            const gt = semver_1.default.valid(containerVersion) && semver_1.default.valid(cauldronContainerVersion)
                ? semver_1.default.gt(containerVersion, cauldronContainerVersion)
                : containerVersion > cauldronContainerVersion;
            if (!gt) {
                throw new Error(`Container version ${containerVersion} is older than ${cauldronContainerVersion}\n${extraErrorMessage}`);
            }
        }
    }
    static noGitOrFilesystemPath(obj, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const packagePaths = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const packagePath of packagePaths) {
            if (packagePath.isFilePath || packagePath.isGitPath) {
                throw new Error(`Found a git or file system path.\n${extraErrorMessage}`);
            }
        }
    }
    static noFileSystemPath(obj, extraErrorMessage = '') {
        const paths = obj instanceof Array ? obj : [obj];
        for (const path of paths) {
            const dependencyPath = ern_core_1.PackagePath.fromString(path);
            if (dependencyPath.isFilePath) {
                throw new Error(`Found a file system path.\n${extraErrorMessage}`);
            }
        }
    }
    static async napDescritorExistsInCauldron(d, extraErrorMessage = '') {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const descriptors = ern_core_1.utils.coerceToAnyAppDescriptorArray(d);
        for (const descriptor of descriptors) {
            const result = await cauldron.isDescriptorInCauldron(descriptor);
            if (!result) {
                throw new Error(`${descriptor.toString()} descriptor does not exist in Cauldron.\n${extraErrorMessage}`);
            }
        }
    }
    static sameNativeApplicationAndPlatform(descriptors, extraErrorMessage = '') {
        const basePathDescriptors = lodash_1.default.map(ern_core_1.utils.coerceToAppVersionDescriptorArray(descriptors), (d) => `${d.name}:${d.platform}`);
        if (lodash_1.default.uniq(basePathDescriptors).length > 1) {
            throw new Error(`Descriptors do not all match the same native application/platform pair.\n${extraErrorMessage}`);
        }
    }
    static async napDescritorDoesNotExistsInCauldron(d, extraErrorMessage = '') {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const descriptor = ern_core_1.utils.coerceToAnyAppDescriptor(d);
        if (await cauldron.isDescriptorInCauldron(descriptor)) {
            throw new Error(`${descriptor} descriptor exist in Cauldron.\n${extraErrorMessage}`);
        }
    }
    static async publishedToNpm(obj, extraErrorMessage = '') {
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const dependency of dependencies) {
            if (!(await ern_core_1.utils.isPublishedToNpm(dependency))) {
                throw new Error(`${dependency} version is not published to npm.\n${extraErrorMessage}`);
            }
        }
    }
    static async miniAppNotInNativeApplicationVersionContainer(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const miniApps = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const miniApp of miniApps) {
            if (await cauldron.isMiniAppInContainer(napDescriptor, miniApp.basePath)) {
                throw new Error(`${miniApp.basePath} MiniApp exists in ${napDescriptor.toString()}.\n${extraErrorMessage}`);
            }
        }
    }
    static async dependencyNotInNativeApplicationVersionContainer(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const dependency of dependencies) {
            if (await cauldron.isNativeDependencyInContainer(napDescriptor, dependency.basePath)) {
                throw new Error(`${dependency.basePath} dependency exists in ${napDescriptor.toString()}.\n${extraErrorMessage}`);
            }
        }
    }
    static async miniAppIsInNativeApplicationVersionContainer(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const miniApps = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const miniApp of miniApps) {
            if (!(await cauldron.isMiniAppInContainer(napDescriptor, miniApp.basePath))) {
                throw new Error(`${miniApp.basePath} MiniApp does not exist in ${napDescriptor.toString()}.\n${extraErrorMessage}`);
            }
        }
    }
    static async dependencyIsInNativeApplicationVersionContainer(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const dependency of dependencies) {
            if (!(await cauldron.getContainerNativeDependency(napDescriptor, dependency.basePath))) {
                throw new Error(`${dependency.basePath} does not exists in ${napDescriptor}.\n${extraErrorMessage}`);
            }
        }
    }
    static async miniAppIsInNativeApplicationVersionContainerWithDifferentVersion(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const miniApps = ern_core_1.utils.coerceToPackagePathArray(obj);
        await Ensure.miniAppIsInNativeApplicationVersionContainer(miniApps, napDescriptor);
        for (const miniApp of miniApps) {
            const miniAppFromCauldron = await cauldron.getContainerMiniApp(napDescriptor, miniApp.basePath);
            const cauldronMiniApp = ern_core_1.PackagePath.fromString(miniAppFromCauldron);
            if (cauldronMiniApp.version === miniApp.version) {
                throw new Error(`${cauldronMiniApp.basePath} is already at version ${miniApp.version || ''} in ${napDescriptor}.\n${extraErrorMessage}`);
            }
        }
    }
    static async dependencyIsInNativeApplicationVersionContainerWithDifferentVersion(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        await Ensure.dependencyIsInNativeApplicationVersionContainer(obj, napDescriptor);
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        for (const dependency of dependencies) {
            const dependencyFromCauldron = await cauldron.getContainerNativeDependency(napDescriptor, dependency.basePath);
            if (dependencyFromCauldron &&
                dependencyFromCauldron.version === dependency.version) {
                throw new Error(`${dependency.basePath} is already at version ${dependencyFromCauldron.version || 'undefined'} in ${napDescriptor.toString()}.\n${extraErrorMessage}`);
            }
        }
    }
    static async dependencyNotInUseByAMiniApp(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        // First let's figure out if any of the MiniApps are using this/these dependency(ies)
        // to make sure that we don't remove any dependency currently used by any MiniApp
        const miniApps = await cauldron.getContainerMiniApps(napDescriptor);
        for (const dependency of dependencies) {
            const miniAppsUsingDependency = await ern_core_1.dependencyLookup.getMiniAppsUsingNativeDependency(miniApps, dependency);
            if ((miniAppsUsingDependency === null || miniAppsUsingDependency === void 0 ? void 0 : miniAppsUsingDependency.length) > 0) {
                let errorMessage = '';
                errorMessage += 'The following MiniApp(s) are using this dependency\n';
                for (const miniApp of miniAppsUsingDependency) {
                    errorMessage += `=> ${miniApp.name}\n`;
                }
                errorMessage +=
                    'You cannot remove a native dependency that is being used by at least a MiniApp\n';
                errorMessage +=
                    'To properly remove this native dependency, you cant either :\n';
                errorMessage +=
                    '- Remove the native dependency from the MiniApp(s) that are using it\n';
                errorMessage +=
                    '- Remove the MiniApps that are using this dependency\n';
                errorMessage +=
                    '- Provide the force flag to this command (if you really now what you are doing !)\n';
                errorMessage += extraErrorMessage;
                throw new Error(errorMessage);
            }
        }
    }
    static async dependencyIsOrphaned(obj, napDescriptor, extraErrorMessage = '') {
        if (!obj) {
            return;
        }
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const lock = await cauldron.getYarnLock(napDescriptor, 'container');
        const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
        if (!lock) {
            return;
        }
        const parser = ern_core_1.YarnLockParser.fromContent(lock.toString());
        let errorMessage = '';
        for (const dependency of dependencies) {
            const tree = parser.buildDependencyTree(ern_core_1.PackagePath.fromString(dependency.basePath));
            if (lodash_1.default.isEmpty(tree) ||
                lodash_1.default.isEmpty(Object.keys(tree).filter((k) => Object.keys(tree[k]).length > 0))) {
                continue;
            }
            errorMessage += `${dependency} is not orphaned. It is used by one or more packages :\n`;
            errorMessage += `${treeify_1.default.asTree(tree, true, true)}\n`;
        }
        if (errorMessage !== '') {
            errorMessage += extraErrorMessage;
            throw new Error(errorMessage);
        }
    }
    static async cauldronIsActive(extraErrorMessage = '') {
        if (!(await (0, ern_cauldron_api_1.getActiveCauldron)({ throwIfNoActiveCauldron: false }))) {
            throw new Error(`There is no active Cauldron\n${extraErrorMessage}`);
        }
    }
    static async pathExist(p, extraErrorMessage = '') {
        return new Promise((resolve, reject) => {
            fs_1.default.exists(p, (exists) => exists
                ? resolve()
                : reject(new Error(`${p} path does not exist.\n${extraErrorMessage}`)));
        });
    }
    static async isFilePath(p, extraErrorMessage = '') {
        return new Promise((resolve, reject) => {
            fs_1.default.stat(p, (err, stats) => {
                if (err) {
                    reject(new Error(`${p} path does not exist.\n${extraErrorMessage}`));
                }
                else {
                    if (stats.isFile()) {
                        resolve();
                    }
                    else {
                        reject(new Error(`${p} is not a file.\n${extraErrorMessage}`));
                    }
                }
            });
        });
    }
    static async isDirectoryPath(p, extraErrorMessage = '') {
        return new Promise((resolve, reject) => {
            fs_1.default.stat(p, (err, stats) => {
                if (err) {
                    reject(new Error(`${p} path does not exist.\n${extraErrorMessage}`));
                }
                else {
                    if (stats.isDirectory()) {
                        resolve();
                    }
                    else {
                        reject(new Error(`${p} is not a directory.\n${extraErrorMessage}`));
                    }
                }
            });
        });
    }
    static checkIfCodePushOptionsAreValid(descriptors, targetBinaryVersion, semVerDescriptor, extraErrorMessage = '') {
        if (targetBinaryVersion && semVerDescriptor) {
            throw new Error('Specify either targetBinaryVersion or semVerDescriptor not both');
        }
        if (targetBinaryVersion && descriptors && descriptors.length > 1) {
            throw new Error('targetBinaryVersion must specify only 1 target native application version for the push');
        }
    }
    static isValidPlatformConfig(key, extraErrorMessage = '') {
        const availablePlatformKeys = () => constants.availableUserConfigKeys.map((e) => e.name);
        if (!availablePlatformKeys().includes(key)) {
            const closestKeyName = (k) => availablePlatformKeys().reduce((acc, cur) => fast_levenshtein_1.default.get(acc, k) > fast_levenshtein_1.default.get(cur, k) ? cur : acc);
            throw new Error(`Configuration key ${key} does not exists. Did you mean ${closestKeyName(key)}?`);
        }
    }
    // Electrode Native currently supports the following versions types
    // to be added to a Container for MiniApps or JS API Implementations
    // dependending of the path type :
    // - File Path      : No intrisic version. Not allowed.
    // - Git Path       : Branch/Tag/Commit SHA
    // - Registry Path  : Fixed (and valid) semantic version. No Ranges.
    static isSupportedMiniAppOrJsApiImplVersion(obj, extraErrorMessage) {
        if (obj) {
            const dependencies = ern_core_1.utils.coerceToPackagePathArray(obj);
            for (const dependency of dependencies) {
                if (dependency.isFilePath) {
                    throw new Error('File Path not supported');
                }
                else if (dependency.isRegistryPath) {
                    if (!dependency.version) {
                        throw new Error(`Missing version for ${dependency}`);
                    }
                    else if (!semver_1.default.valid(dependency.version)) {
                        throw new Error(`Unsupported version ${dependency.version} for ${dependency.basePath}`);
                    }
                }
                else if (!dependency.version) {
                    // git path
                    throw new Error(`Missing version for ${dependency}`);
                }
            }
        }
    }
    static isContainerPath(path, extraErrorMessage = '') {
        if (!fs_1.default.existsSync((0, ern_container_gen_1.getContainerMetadataPath)(path))) {
            throw new Error(`${path} is not a path to a Container\n${extraErrorMessage}`);
        }
    }
    static isEnvVariableDefined(envVarName, extraErrorMessage = '') {
        if (!process.env[envVarName]) {
            throw new Error(`${envVarName} is not defined\n${extraErrorMessage}`);
        }
    }
    static async manifestIdExists(manifestId, extraErrorMessage = '') {
        if (!(await ern_core_1.manifest.hasManifestId(manifestId))) {
            throw new Error(`${manifestId} id not found in the Manifest(s)\n${extraErrorMessage}`);
        }
    }
    static async bundleStoreUrlSetInCauldron(extraErrorMessage = '') {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const bundleStoreConfig = await cauldron.getBundleStoreConfig();
        if (!bundleStoreConfig || !bundleStoreConfig.url) {
            throw new Error(`bundleStore url not set in Cauldron\n${extraErrorMessage}`);
        }
    }
    static bundleStoreAccessKeyIsSet(extraErrorMessage = '') {
        if (!ern_core_1.config.get('bundlestore-accesskey')) {
            throw new Error(`bundlestore-accesskey is not set in configuration\n${extraErrorMessage}`);
        }
    }
    static async metroServerIsNotRunning(host, port, extraErrorMessage) {
        const metroServerUrl = `http://${host}:${port}`;
        try {
            await got_1.default.get(metroServerUrl);
        }
        catch (err) {
            return;
        }
        throw new Error(`Metro server is running on ${metroServerUrl}\n${extraErrorMessage}`);
    }
}
exports.default = Ensure;
//# sourceMappingURL=Ensure.js.map