"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildIosRunner = void 0;
const ern_core_1 = require("ern-core");
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
async function buildIosRunner(pathToIosRunner, udid) {
    return new Promise((resolve, reject) => {
        const extraBuildOptions = fs_extra_1.default.existsSync(path_1.default.join(pathToIosRunner, 'ErnRunner.xcworkspace'))
            ? [`-workspace`, `ErnRunner.xcworkspace`]
            : [];
        const xcodebuildProc = (0, child_process_1.spawn)('xcodebuild', [
            ...extraBuildOptions,
            `-scheme`,
            'ErnRunner',
            'build',
            `-destination`,
            `id=${udid}`,
            `SYMROOT=${pathToIosRunner}/build`,
        ], { cwd: pathToIosRunner });
        xcodebuildProc.stdout.on('data', (data) => {
            ern_core_1.log.debug(data.toString());
        });
        xcodebuildProc.stderr.on('data', (data) => {
            ern_core_1.log.debug(data.toString());
        });
        xcodebuildProc.on('close', (code) => {
            code === 0
                ? resolve()
                : reject(new Error(`iOS Runner build failed [xcbuild exit code ${code}].
To troubleshoot this build failure, we recommend building the Runner iOS project from Xcode.
You can open the Runner project in Xcode manually or by running 'open ios/ErnRunner.xcodeproj'.
Building the Runner from Xcode will provide more meaningful error reporting that can be of help
to pinpoint the cause of the build failure.`));
        });
    });
}
exports.buildIosRunner = buildIosRunner;
//# sourceMappingURL=buildIosRunner.js.map