"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.orderPackagePaths = exports.getCodePushAppName = exports.removeZeroPatchDigit = exports.applyVersionModifiers = exports.buildCodePushTargetBinaryVersion = exports.performCodePushOtaUpdate = exports.performCodePushPromote = exports.performCodePushPatch = void 0;
const ern_core_1 = require("ern-core");
const ern_composite_gen_1 = require("ern-composite-gen");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const compatibility = __importStar(require("./compatibility"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const shelljs_1 = __importDefault(require("shelljs"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = __importDefault(require("semver"));
async function performCodePushPatch(napDescriptor, deploymentName, label, { description, isDisabled, isMandatory, rollout, } = {}) {
    let cauldron;
    try {
        cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        await cauldron.beginTransaction();
        const appName = await getCodePushAppName(napDescriptor);
        const codePushSdk = (0, ern_core_1.getCodePushSdk)();
        description = description || '';
        await codePushSdk.patch(appName, deploymentName, {
            description,
            isDisabled,
            isMandatory,
            label,
            rollout,
        });
        await cauldron.updateCodePushEntry(napDescriptor, {
            deploymentName,
            description,
            isDisabled,
            isMandatory,
            label,
            rollout,
        });
        await cauldron.commitTransaction(`CodePush patched ${napDescriptor.toString()} ${deploymentName} ${label}`);
    }
    catch (e) {
        if (cauldron) {
            await cauldron.discardTransaction();
        }
        ern_core_1.log.error(`[performCodePushPatch] ${e}`);
        throw e;
    }
}
exports.performCodePushPatch = performCodePushPatch;
async function performCodePushPromote(sourceNapDescriptor, targetNapDescriptors, sourceDeploymentName, targetDeploymentName, { description, disableDuplicateReleaseError = false, force = false, label, mandatory, reuseReleaseBinaryVersion, rollout, skipNativeDependenciesVersionAlignedCheck = false, targetBinaryVersion, } = {}) {
    let cauldron;
    try {
        if (reuseReleaseBinaryVersion && targetBinaryVersion) {
            throw new Error(`sameBinaryVersionAsRelease and targetBinaryVersion options are mutually exclusive`);
        }
        const codePushSdk = (0, ern_core_1.getCodePushSdk)();
        cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        await cauldron.beginTransaction();
        const cauldronCommitMessage = [
            `CodePush release promotion of ${sourceNapDescriptor.toString()} ${sourceDeploymentName} to ${targetDeploymentName} of`,
        ];
        for (const targetNapDescriptor of targetNapDescriptors) {
            if (!targetNapDescriptor.version) {
                throw new Error(`Missing version in ${targetNapDescriptor.toString()}`);
            }
            const codePushEntrySource = await cauldron.getCodePushEntry(sourceNapDescriptor, sourceDeploymentName, { label });
            if (!codePushEntrySource) {
                throw new Error(`No CodePush entry found in Cauldron matching [desc: ${sourceNapDescriptor} dep: ${sourceDeploymentName} label: ${label || 'latest'}`);
            }
            const miniApps = lodash_1.default.map(codePushEntrySource.miniapps, (miniapp) => ern_core_1.PackagePath.fromString(miniapp));
            const jsApiImpls = lodash_1.default.map(codePushEntrySource.jsApiImpls, (jsapiimpl) => ern_core_1.PackagePath.fromString(jsapiimpl));
            // If sourceNapDescriptor.version === targetNapDescriptor.version skip the compatibility check
            // As the compatibility check would have already happened as part of the code-push release
            // If skipNativeDependenciesVersionAlignedCheck is set to true
            // and sourceNapDescriptor and targetNapDescriptor versions are different, explicity skip the check
            // would be required for patch version promotion
            if (sourceNapDescriptor &&
                sourceNapDescriptor.version !== targetNapDescriptor.version &&
                !skipNativeDependenciesVersionAlignedCheck) {
                const nativeDependenciesVersionAligned = await compatibility.areCompatible(miniApps, targetNapDescriptor);
                if (!nativeDependenciesVersionAligned && force) {
                    ern_core_1.log.warn('Native dependencies versions are not aligned but ignoring due to the use of force flag');
                }
                else if (!nativeDependenciesVersionAligned && !force) {
                    throw new Error('Native dependencies versions of MiniApps are not aligned. Relaunch the operation with the force flag if you wish to ignore.');
                }
            }
            const appName = await getCodePushAppName(sourceNapDescriptor);
            targetBinaryVersion = targetBinaryVersion
                ? targetBinaryVersion
                : reuseReleaseBinaryVersion
                    ? codePushEntrySource.metadata.appVersion
                    : await buildCodePushTargetBinaryVersion(targetNapDescriptor, targetDeploymentName);
            description = description || '';
            const codePushResponse = await ern_core_1.kax
                .task(`Promoting release to ${targetBinaryVersion}`)
                .run(codePushSdk.promote(appName, sourceDeploymentName, targetDeploymentName, {
                appVersion: targetBinaryVersion,
                description,
                isMandatory: !!mandatory,
                label,
                rollout,
            }, disableDuplicateReleaseError));
            if (codePushResponse) {
                const sourceYarnLockId = await cauldron.getYarnLockId(sourceNapDescriptor, sourceDeploymentName);
                if (!sourceYarnLockId) {
                    ern_core_1.log.error(`No yarn.lock was found in source deployment [${sourceDeploymentName} for ${sourceNapDescriptor.toString()}]`);
                    ern_core_1.log.error(`Skipping promotion to ${targetNapDescriptor.toString()}`);
                    continue;
                }
                await cauldron.updateYarnLockId(targetNapDescriptor, targetDeploymentName, sourceYarnLockId);
                await cauldron.addCodePushEntry(targetNapDescriptor, {
                    appVersion: codePushResponse.appVersion,
                    deploymentName: targetDeploymentName,
                    description: codePushResponse.description,
                    isMandatory: codePushResponse.isMandatory,
                    label: codePushResponse.label,
                    promotedFromLabel: codePushEntrySource.metadata.label,
                    releaseMethod: codePushResponse.releaseMethod,
                    releasedBy: codePushResponse.releasedBy,
                    rollout: codePushResponse.rollout,
                    size: codePushResponse.size,
                }, miniApps, jsApiImpls || []);
                // Copy source map if a source map store server is configured
                const sourcemapStoreConfig = await cauldron.getSourceMapStoreConfig();
                if (sourcemapStoreConfig) {
                    try {
                        const sdk = new ern_core_1.SourceMapStoreSdk(sourcemapStoreConfig.url);
                        await ern_core_1.kax
                            .task(`Copying source map in source map store [${sourcemapStoreConfig.url}]`)
                            .run(sdk.copyCodePushSourceMap({
                            deploymentName: sourceDeploymentName,
                            descriptor: sourceNapDescriptor,
                            label: codePushEntrySource.metadata.label,
                            toDeploymentName: targetDeploymentName,
                            toLabel: codePushResponse.label,
                            toVersion: targetNapDescriptor.version,
                        }));
                    }
                    catch (e) {
                        ern_core_1.log.error(`Source map copy failed : ${e}`);
                    }
                }
                cauldronCommitMessage.push(`- ${targetNapDescriptor.toString()}`);
            }
            await ern_core_1.kax
                .task('Updating Cauldron')
                .run(cauldron.commitTransaction(cauldronCommitMessage));
        }
    }
    catch (e) {
        if (cauldron) {
            await cauldron.discardTransaction();
        }
        ern_core_1.log.error(`[performCodePushPromote] ${e}`);
        throw e;
    }
}
exports.performCodePushPromote = performCodePushPromote;
async function performCodePushOtaUpdate(napDescriptor, deploymentName, miniApps, jsApiImpls, { baseComposite, codePushIsMandatoryRelease = false, codePushRolloutPercentage, description, disableDuplicateReleaseError = false, force = false, pathToYarnLock, sourceMapOutput, targetBinaryVersion, } = {}) {
    var _a, _b;
    let cauldron;
    try {
        const codePushSdk = (0, ern_core_1.getCodePushSdk)();
        cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const plugins = await cauldron.getNativeDependencies(napDescriptor);
        const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(napDescriptor);
        baseComposite =
            baseComposite !== null && baseComposite !== void 0 ? baseComposite : ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
        await cauldron.beginTransaction();
        const codePushPlugin = lodash_1.default.find(plugins, (p) => p.name === 'react-native-code-push');
        if (!codePushPlugin) {
            throw new Error('react-native-code-push plugin is not in native app !');
        }
        const reactNative = lodash_1.default.find(plugins, (p) => p.name === 'react-native');
        if (!codePushPlugin) {
            throw new Error('react-native is not in native app !');
        }
        const tmpWorkingDir = (0, ern_core_1.createTmpDir)();
        const miniAppsNativeDependenciesVersionAligned = await compatibility.areCompatible(miniApps, napDescriptor);
        if (!miniAppsNativeDependenciesVersionAligned && force) {
            ern_core_1.log.warn('Native dependencies versions of MiniApps are not aligned but ignoring due to the use of force flag');
        }
        else if (!miniAppsNativeDependenciesVersionAligned && !force) {
            throw new Error('Native dependencies versions of MiniApps are not aligned. Relaunch the operation with the force flag if you wish to ignore.');
        }
        const latestCodePushedMiniApps = await cauldron.getCodePushMiniApps(napDescriptor, deploymentName);
        const latestCodePushedJsApiImpls = await cauldron.getCodePushJsApiImpls(napDescriptor, deploymentName);
        // We need to include, in this CodePush bundle, all the MiniApps and JS API implementations that were part
        // of the previous CodePush. We will override versions of the MiniApps and JS API implementations with
        // the one provided to this function, and keep other ones intact.
        // For example, if previous CodePush bundle was containing first-miniapp@1.0.0 and
        // second-miniapp@1.0.0 and this method is called to CodePush first-miniapp@2.0.0, then
        // the bundle we will push will container first-miniapp@2.0.0 and second-miniapp@1.0.0.
        // If this the first ever CodePush bundle for this specific native application version
        // then the reference miniapp versions are the one from the container.
        let referenceMiniAppsToCodePush = latestCodePushedMiniApps;
        if (!referenceMiniAppsToCodePush ||
            referenceMiniAppsToCodePush.length === 0) {
            referenceMiniAppsToCodePush = await cauldron.getContainerMiniApps(napDescriptor);
        }
        let referenceJsApiImplsToCodePush = latestCodePushedJsApiImpls;
        if (!referenceJsApiImplsToCodePush ||
            referenceJsApiImplsToCodePush.length === 0) {
            referenceJsApiImplsToCodePush = await cauldron.getContainerJsApiImpls(napDescriptor);
        }
        const miniAppsToBeCodePushedUnordered = lodash_1.default.unionBy(miniApps, referenceMiniAppsToCodePush, (x) => x.basePath);
        // Reorder MiniApps to preserve current order set in Cauldron
        const miniAppsToBeCodePushed = orderPackagePaths(referenceMiniAppsToCodePush, miniAppsToBeCodePushedUnordered);
        const jsApiImplsToBeCodePushedUnordered = lodash_1.default.unionBy(jsApiImpls, referenceJsApiImplsToCodePush, (x) => x.basePath);
        // Reorder JS API implementations to preserve current order set in Cauldron
        const jsApiImplsToBeCodePushed = orderPackagePaths(referenceJsApiImplsToCodePush, jsApiImplsToBeCodePushedUnordered);
        const pathsToMiniAppsToBeCodePushed = lodash_1.default.map(miniAppsToBeCodePushed, (m) => ern_core_1.PackagePath.fromString(m.toString()));
        const pathToJsApiImplsToBeCodePushed = lodash_1.default.map(jsApiImplsToBeCodePushed, (j) => ern_core_1.PackagePath.fromString(j.toString()));
        const codePushConfig = await cauldron.getCodePushConfig(napDescriptor);
        if (codePushConfig && codePushConfig.bypassYarnLock) {
            pathToYarnLock = undefined;
            ern_core_1.log.debug('Bypassing yarn.lock usage as bypassYarnLock flag is set in Cauldron config');
        }
        const composite = await ern_core_1.kax.task('Generating composite module').run(ern_composite_gen_1.GeneratedComposite.generate({
            baseComposite,
            jsApiImplDependencies: pathToJsApiImplsToBeCodePushed,
            miniApps: pathsToMiniAppsToBeCodePushed,
            outDir: tmpWorkingDir,
            pathToYarnLock,
            resolutions: compositeGenConfig && compositeGenConfig.resolutions,
        }));
        const bundleOutputDirectory = path_1.default.join(tmpWorkingDir, 'bundleOut');
        shelljs_1.default.mkdir('-p', bundleOutputDirectory);
        const platform = napDescriptor.platform || '';
        const bundleOutputPath = platform === 'android'
            ? path_1.default.join(bundleOutputDirectory, 'index.android.bundle')
            : path_1.default.join(bundleOutputDirectory, 'MiniApp.jsbundle');
        sourceMapOutput = sourceMapOutput || path_1.default.join((0, ern_core_1.createTmpDir)(), 'index.map');
        const entryFile = fs_extra_1.default.existsSync(path_1.default.join(tmpWorkingDir, `index.${platform}.js`))
            ? `index.${platform}.js`
            : 'index.js';
        const bundlingResult = await ern_core_1.kax
            .task('Generating composite bundle for miniapps')
            .run(ern_core_1.reactnative.bundle({
            assetsDest: bundleOutputDirectory,
            bundleOutput: bundleOutputPath,
            dev: false,
            entryFile,
            platform,
            resetCache: true,
            sourceMapOutput,
            workingDir: tmpWorkingDir,
        }));
        if (platform === 'android') {
            const conf = await cauldron.getContainerGeneratorConfig(napDescriptor);
            const isHermesEnabled = ((_a = conf === null || conf === void 0 ? void 0 : conf.androidConfig) === null || _a === void 0 ? void 0 : _a.jsEngine) === 'hermes';
            if (isHermesEnabled) {
                const hermesVersion = (_b = conf.androidConfig.hermesVersion) !== null && _b !== void 0 ? _b : ern_core_1.android.getDefaultHermesVersion(reactNative === null || reactNative === void 0 ? void 0 : reactNative.version);
                const hermesCli = await ern_core_1.kax
                    .task(`Installing hermes-engine@${hermesVersion}`)
                    .run(ern_core_1.HermesCli.fromVersion(hermesVersion));
                const res = await ern_core_1.kax
                    .task('Compiling JS bundle to Hermes bytecode')
                    .run(hermesCli.compileReleaseBundle({
                    bundleSourceMapPath: sourceMapOutput,
                    compositePath: tmpWorkingDir,
                    jsBundlePath: bundleOutputPath,
                }));
                bundlingResult.isHermesBundle = true;
            }
        }
        const appName = await getCodePushAppName(napDescriptor);
        targetBinaryVersion =
            targetBinaryVersion ||
                (await buildCodePushTargetBinaryVersion(napDescriptor, deploymentName));
        ern_core_1.log.info(`Target Binary version : ${targetBinaryVersion}`);
        description = description || '';
        const codePushResponse = await ern_core_1.kax
            .task('Releasing bundle through CodePush')
            .run(codePushSdk.releaseReact(appName, deploymentName, bundleOutputDirectory, targetBinaryVersion, {
            description,
            isMandatory: codePushIsMandatoryRelease,
            rollout: codePushRolloutPercentage,
        }, disableDuplicateReleaseError));
        if (codePushResponse) {
            await cauldron.addCodePushEntry(napDescriptor, {
                appVersion: codePushResponse.appVersion,
                deploymentName,
                description: codePushResponse.description,
                isMandatory: codePushResponse.isMandatory,
                label: codePushResponse.label,
                releaseMethod: codePushResponse.releaseMethod,
                releasedBy: codePushResponse.releasedBy,
                rollout: codePushResponse.rollout,
                size: codePushResponse.size,
            }, miniAppsToBeCodePushed, jsApiImplsToBeCodePushed);
            const pathToNewYarnLock = path_1.default.join(tmpWorkingDir, 'yarn.lock');
            if (await fs_extra_1.default.pathExists(pathToNewYarnLock)) {
                await ern_core_1.kax
                    .task('Adding yarn.lock to Cauldron')
                    .run(cauldron.addOrUpdateYarnLock(napDescriptor, deploymentName, pathToNewYarnLock));
            }
            // Upload source map if a source map store server is configured
            const sourcemapStoreConfig = await cauldron.getSourceMapStoreConfig();
            if (sourcemapStoreConfig) {
                try {
                    const sdk = new ern_core_1.SourceMapStoreSdk(sourcemapStoreConfig.url);
                    await ern_core_1.kax
                        .task(`Uploading source map to source map store [${sourcemapStoreConfig.url}]`)
                        .run(sdk.uploadCodePushSourceMap({
                        deploymentName,
                        descriptor: napDescriptor,
                        label: codePushResponse.label,
                        sourceMapPath: sourceMapOutput,
                    }));
                }
                catch (e) {
                    ern_core_1.log.error(`Source map upload failed : ${e}`);
                }
            }
            // Upload source map to bugsnag if configured
            const bugsnagConfig = await cauldron.getBugsnagConfig(napDescriptor);
            if (bugsnagConfig) {
                try {
                    const { apiKey } = bugsnagConfig;
                    const [minifiedFile, minifiedUrl, projectRoot, sourceMap] = [
                        await fs_extra_1.default.realpath(bundleOutputPath),
                        path_1.default.basename(bundleOutputPath),
                        await fs_extra_1.default.realpath(path_1.default.join(composite.path, 'node_modules')),
                        await fs_extra_1.default.realpath(sourceMapOutput),
                    ];
                    const compositeMiniAppsPackages = await composite.getMiniAppsPackages();
                    await (0, ern_core_1.bugsnagUpload)({
                        apiKey,
                        minifiedFile,
                        minifiedUrl,
                        projectRoot,
                        sourceMap,
                        uploadNodeModules: false,
                        uploadSources: false,
                    });
                }
                catch (e) {
                    ern_core_1.log.error(`Bugsnag upload failed : ${e}`);
                }
            }
            await cauldron.commitTransaction(`CodePush release for ${napDescriptor.toString()} ${deploymentName}`);
        }
    }
    catch (e) {
        if (cauldron) {
            await cauldron.discardTransaction();
        }
        ern_core_1.log.error(`performCodePushOtaUpdate ${e}`);
        throw e;
    }
}
exports.performCodePushOtaUpdate = performCodePushOtaUpdate;
async function buildCodePushTargetBinaryVersion(napDescriptor, deploymentName) {
    if (!napDescriptor.version) {
        throw new Error(`Native application descriptor ${napDescriptor} does not contain a version !`);
    }
    let targetBinaryVersion = napDescriptor.version;
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    const codePushConfig = await cauldron.getCodePushConfig(napDescriptor);
    if (codePushConfig) {
        const { versionModifiers, trimZeroPatchDigit } = codePushConfig;
        targetBinaryVersion = versionModifiers
            ? applyVersionModifiers({
                deploymentName,
                targetBinaryVersion,
                versionModifiers,
            })
            : targetBinaryVersion;
        targetBinaryVersion =
            trimZeroPatchDigit &&
                semver_1.default.valid(targetBinaryVersion) &&
                semver_1.default.patch(targetBinaryVersion) === 0
                ? removeZeroPatchDigit({
                    targetBinaryVersion,
                })
                : targetBinaryVersion;
    }
    return targetBinaryVersion;
}
exports.buildCodePushTargetBinaryVersion = buildCodePushTargetBinaryVersion;
function applyVersionModifiers({ deploymentName, targetBinaryVersion, versionModifiers, }) {
    const versionModifier = lodash_1.default.find(versionModifiers, (m) => m.deploymentName === deploymentName);
    return versionModifier
        ? targetBinaryVersion.replace(/(.+)/, versionModifier.modifier)
        : targetBinaryVersion;
}
exports.applyVersionModifiers = applyVersionModifiers;
function removeZeroPatchDigit({ targetBinaryVersion, }) {
    return semver_1.default.prerelease(targetBinaryVersion)
        ? `${semver_1.default.major(targetBinaryVersion)}.${semver_1.default.minor(targetBinaryVersion)}-${semver_1.default.prerelease(targetBinaryVersion).join('.')}`
        : `${semver_1.default.major(targetBinaryVersion)}.${semver_1.default.minor(targetBinaryVersion)}`;
}
exports.removeZeroPatchDigit = removeZeroPatchDigit;
async function getCodePushAppName(napDescriptor) {
    var _a, _b, _c, _d, _e;
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    const codePushConfig = await cauldron.getCodePushConfig(napDescriptor);
    const cauldronCodepushAppName = (_d = (_c = (_b = (_a = (await cauldron.getAllNativeApps())
        .find((app) => app.name === napDescriptor.name)) === null || _a === void 0 ? void 0 : _a.platforms.find((platform) => platform.name === napDescriptor.platform)) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c.codePush) === null || _d === void 0 ? void 0 : _d.appName;
    return cauldronCodepushAppName
        ? cauldronCodepushAppName
        : (_e = codePushConfig === null || codePushConfig === void 0 ? void 0 : codePushConfig.appName) !== null && _e !== void 0 ? _e : `${napDescriptor.name}${napDescriptor.platform === 'ios' ? 'Ios' : 'Android'}`;
}
exports.getCodePushAppName = getCodePushAppName;
// Reorder packages to preserve current order as set in Cauldron
function orderPackagePaths(referencePackagePaths, packagePaths) {
    var _a;
    const packages = [];
    for (const m of referencePackagePaths) {
        const x = (_a = lodash_1.default.find(packagePaths, (p) => p.basePath === m.basePath)) !== null && _a !== void 0 ? _a : m;
        packages.push(x);
    }
    // Add all new packages if any
    const newPackages = lodash_1.default.differenceBy(packagePaths, referencePackagePaths, (p) => p.basePath);
    packages.push(...newPackages);
    return packages;
}
exports.orderPackagePaths = orderPackagePaths;
//# sourceMappingURL=codepush.js.map