"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logDependenciesTree = exports.logMismatchingDependenciesTree = exports.logResolvedDependenciesTree = exports.logResolvedAndMismatchingDependenciesTree = exports.validateCompositeNativeDependencies = exports.runCauldronCompositeGen = exports.runLocalCompositeGen = void 0;
const ern_composite_gen_1 = require("ern-composite-gen");
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const treeify_1 = __importDefault(require("treeify"));
const constants = __importStar(require("./constants"));
const path_1 = __importDefault(require("path"));
async function runLocalCompositeGen({ baseComposite, jsApiImpls, metroExtraNodeModules, miniApps, outDir, resolutions, }) {
    try {
        const composite = await ern_core_1.kax.task('Generating Composite').run(ern_composite_gen_1.GeneratedComposite.generate({
            baseComposite,
            jsApiImplDependencies: jsApiImpls,
            metroExtraNodeModules,
            miniApps,
            outDir,
            resolutions,
        }));
        await validateCompositeNativeDependencies(composite);
        return composite;
    }
    catch (e) {
        ern_core_1.log.error(`runLocalCompositeGen failed: ${e}`);
        throw e;
    }
}
exports.runLocalCompositeGen = runLocalCompositeGen;
// Run container generator using the Cauldron, given a native application descriptor
async function runCauldronCompositeGen(napDescriptor, { baseComposite, outDir, favorGitBranches, }) {
    try {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(napDescriptor);
        baseComposite =
            baseComposite ||
                ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                    ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
        const miniapps = await cauldron.getContainerMiniApps(napDescriptor, {
            favorGitBranches,
        });
        const jsApiImpls = await cauldron.getContainerJsApiImpls(napDescriptor);
        // bypassYarnLock to move into compositeGen config
        const containerGenConfig = await cauldron.getContainerGeneratorConfig(napDescriptor);
        let pathToYarnLock;
        if (!containerGenConfig || !containerGenConfig.bypassYarnLock) {
            pathToYarnLock = await cauldron.getPathToYarnLock(napDescriptor, constants.CONTAINER_YARN_KEY);
        }
        else {
            ern_core_1.log.debug('Bypassing yarn.lock usage as bypassYarnLock flag is set in Cauldron config');
        }
        const composite = await ern_core_1.kax.task('Generating Composite').run(ern_composite_gen_1.GeneratedComposite.generate({
            baseComposite,
            jsApiImplDependencies: jsApiImpls,
            miniApps: miniapps,
            outDir,
            pathToYarnLock,
            resolutions: compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.resolutions,
        }));
        await validateCompositeNativeDependencies(composite);
        return composite;
    }
    catch (e) {
        ern_core_1.log.error(`runCauldronCompositeGen failed: ${e}`);
        throw e;
    }
}
exports.runCauldronCompositeGen = runCauldronCompositeGen;
async function validateCompositeNativeDependencies(composite) {
    // Validate composite native dependencies
    const resolution = await composite.getResolvedNativeDependencies();
    if (resolution.pluginsWithMismatchingVersions.length > 0) {
        logMismatchingDependenciesTree(composite, resolution);
        throw new Error(`The following plugins are using incompatible versions:
     ${resolution.pluginsWithMismatchingVersions.toString()}`);
    }
    try {
        logResolvedDependenciesTree(composite, resolution);
    }
    catch (e) {
        ern_core_1.log.error(e);
    }
}
exports.validateCompositeNativeDependencies = validateCompositeNativeDependencies;
function logResolvedAndMismatchingDependenciesTree(composite, resolution) {
    logResolvedDependenciesTree(composite, resolution);
    if (resolution.pluginsWithMismatchingVersions.length > 0) {
        logMismatchingDependenciesTree(composite, resolution);
    }
}
exports.logResolvedAndMismatchingDependenciesTree = logResolvedAndMismatchingDependenciesTree;
function logResolvedDependenciesTree(composite, resolution) {
    const parser = ern_core_1.YarnLockParser.fromPath(path_1.default.join(composite.path, 'yarn.lock'));
    ern_core_1.log.debug('[ == RESOLVED NATIVE DEPENDENCIES ==]');
    logDependenciesTree(parser, resolution.resolved.map((x) => ern_core_1.PackagePath.fromString(x.name)), 'debug');
}
exports.logResolvedDependenciesTree = logResolvedDependenciesTree;
function logMismatchingDependenciesTree(composite, resolution) {
    const parser = ern_core_1.YarnLockParser.fromPath(path_1.default.join(composite.path, 'yarn.lock'));
    ern_core_1.log.error('[ == MISMATCHING NATIVE DEPENDENCIES ==]');
    logDependenciesTree(parser, resolution.pluginsWithMismatchingVersions.map(ern_core_1.PackagePath.fromString), 'error');
}
exports.logMismatchingDependenciesTree = logMismatchingDependenciesTree;
function logDependenciesTree(parser, deps, logLevel) {
    for (const dep of deps) {
        const depTree = parser.buildDependencyTree(dep);
        logLevel === 'debug'
            ? ern_core_1.log.debug(treeify_1.default.asTree(depTree, true, true))
            : ern_core_1.log.error(treeify_1.default.asTree(depTree, true, true));
    }
}
exports.logDependenciesTree = logDependenciesTree;
//# sourceMappingURL=composite.js.map