"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCaudronBundleGen = exports.runCauldronContainerGen = exports.runLocalContainerGen = void 0;
const ern_container_gen_1 = require("ern-container-gen");
const ern_container_gen_android_1 = require("ern-container-gen-android");
const ern_container_gen_ios_1 = require("ern-container-gen-ios");
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const constants = __importStar(require("./constants"));
// Run container generator locally, without relying on the Cauldron, given a list of miniapp packages
// The string used to represent a miniapp package can be anything supported by `yarn add` command
// For example, the following miniapp strings are all valid
// FROM NPM => react-native-miniapp@1.2.3
// FROM GIT => git@github.com:org/test-miniapp.git
// FROM FS  => file:/home/user/test-miniapp
async function runLocalContainerGen(platform, composite, { outDir = ern_core_1.Platform.getContainerGenOutDirectory(platform), ignoreRnpmAssets = false, jsMainModuleName, extra, sourceMapOutput, devJsBundle, resetCache, }) {
    var _a, _b, _c, _d;
    try {
        const generator = getGeneratorForPlatform(platform);
        return ern_core_1.kax.task('Generating Container').run(generator.generate({
            androidConfig: (_a = extra === null || extra === void 0 ? void 0 : extra.androidConfig) !== null && _a !== void 0 ? _a : {},
            composite,
            devJsBundle,
            hooks: (_c = (_b = extra === null || extra === void 0 ? void 0 : extra.containerGenerator) === null || _b === void 0 ? void 0 : _b.hooks) !== null && _c !== void 0 ? _c : {},
            ignoreRnpmAssets,
            iosConfig: (_d = extra === null || extra === void 0 ? void 0 : extra.iosConfig) !== null && _d !== void 0 ? _d : {},
            jsMainModuleName,
            outDir,
            plugins: [],
            resetCache,
            sourceMapOutput,
            targetPlatform: platform,
        }));
    }
    catch (e) {
        ern_core_1.log.error(`runLocalContainerGen failed: ${e}`);
        throw e;
    }
}
exports.runLocalContainerGen = runLocalContainerGen;
// Run container generator using the Cauldron, given a native application descriptor
async function runCauldronContainerGen(napDescriptor, composite, { devJsBundle, jsMainModuleName, outDir, resetCache, sourceMapOutput, } = {}) {
    var _a;
    try {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        if (!napDescriptor.platform) {
            throw new Error(`${napDescriptor} does not specify a platform`);
        }
        const plugins = await composite.getInjectableNativeDependencies(napDescriptor.platform);
        const platform = napDescriptor.platform;
        const containerGeneratorConfig = await cauldron.getContainerGeneratorConfig(napDescriptor);
        const generator = getGeneratorForPlatform(platform);
        const containerGenResult = await ern_core_1.kax
            .task(`Generating Container for ${napDescriptor.toString()}`)
            .run(generator.generate({
            androidConfig: containerGeneratorConfig === null || containerGeneratorConfig === void 0 ? void 0 : containerGeneratorConfig.androidConfig,
            composite,
            devJsBundle: devJsBundle === undefined
                ? containerGeneratorConfig === null || containerGeneratorConfig === void 0 ? void 0 : containerGeneratorConfig.devJsBundle
                : devJsBundle,
            hooks: (_a = containerGeneratorConfig === null || containerGeneratorConfig === void 0 ? void 0 : containerGeneratorConfig.hooks) !== null && _a !== void 0 ? _a : {},
            ignoreRnpmAssets: containerGeneratorConfig &&
                containerGeneratorConfig.ignoreRnpmAssets,
            iosConfig: containerGeneratorConfig === null || containerGeneratorConfig === void 0 ? void 0 : containerGeneratorConfig.iosConfig,
            jsMainModuleName,
            outDir: outDir || ern_core_1.Platform.getContainerGenOutDirectory(platform),
            plugins,
            resetCache: resetCache !== null && resetCache !== void 0 ? resetCache : containerGeneratorConfig === null || containerGeneratorConfig === void 0 ? void 0 : containerGeneratorConfig.resetCache,
            sourceMapOutput,
            targetPlatform: platform,
        }));
        return containerGenResult;
    }
    catch (e) {
        ern_core_1.log.error(`runCauldronContainerGen failed: ${e}`);
        throw e;
    }
}
exports.runCauldronContainerGen = runCauldronContainerGen;
async function runCaudronBundleGen(napDescriptor, { baseComposite, compositeDir, outDir, resetCache, resolutions, }) {
    try {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(napDescriptor);
        baseComposite =
            baseComposite ||
                ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                    ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
        const miniapps = await cauldron.getContainerMiniApps(napDescriptor);
        const jsApiImpls = await cauldron.getContainerJsApiImpls(napDescriptor);
        const containerGenConfig = await cauldron.getContainerGeneratorConfig(napDescriptor);
        let pathToYarnLock;
        if (!containerGenConfig || !containerGenConfig.bypassYarnLock) {
            pathToYarnLock = await cauldron.getPathToYarnLock(napDescriptor, constants.CONTAINER_YARN_KEY);
        }
        else {
            ern_core_1.log.debug('Bypassing yarn.lock usage as bypassYarnLock flag is set in Cauldron config');
        }
        if (!napDescriptor.platform) {
            throw new Error(`${napDescriptor} does not specify a platform`);
        }
        return ern_core_1.kax.task('Bundling MiniApps').run((0, ern_container_gen_1.bundleMiniApps)(miniapps, compositeDir || (0, ern_core_1.createTmpDir)(), outDir, napDescriptor.platform, {
            baseComposite,
            jsApiImplDependencies: jsApiImpls,
            pathToYarnLock: pathToYarnLock || undefined,
            resetCache,
            resolutions,
        }));
    }
    catch (e) {
        ern_core_1.log.error(`runCauldronBundleGen failed: ${e}`);
        throw e;
    }
}
exports.runCaudronBundleGen = runCaudronBundleGen;
function getGeneratorForPlatform(platform) {
    switch (platform) {
        case 'android':
            return new ern_container_gen_android_1.AndroidGenerator();
        case 'ios':
            return new ern_container_gen_ios_1.IosGenerator();
        default:
            throw new Error(`Unsupported platform : ${platform}`);
    }
}
//# sourceMappingURL=container.js.map