"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.launchOnDevice = void 0;
const ern_core_1 = require("ern-core");
const buildIosRunner_1 = require("./buildIosRunner");
const child_process_1 = require("child_process");
async function launchOnDevice(pathToIosRunner, devices, { launchArgs, launchEnvVars, } = {}) {
    const iPhoneDevice = await ern_core_1.ios.askUserToSelectAniPhoneDevice(devices);
    ern_core_1.shell.pushd(pathToIosRunner);
    try {
        await ern_core_1.kax
            .task('Building iOS Runner project')
            .run((0, buildIosRunner_1.buildIosRunner)(pathToIosRunner, iPhoneDevice.udid));
        const kaxDeployTask = ern_core_1.kax.task(`Installing iOS Runner on ${iPhoneDevice.name}`);
        try {
            const iosDeployInstallArgs = [
                '--bundle',
                `${pathToIosRunner}/build/Debug-iphoneos/ErnRunner.app`,
                '--id',
                iPhoneDevice.udid,
                '--justlaunch',
            ];
            if (launchArgs) {
                iosDeployInstallArgs.push('--args');
                iosDeployInstallArgs.push(launchArgs.split(' '));
            }
            if (launchEnvVars) {
                iosDeployInstallArgs.push('--envs');
                iosDeployInstallArgs.push(launchEnvVars.split(' '));
            }
            const iosDeployOutput = (0, child_process_1.spawnSync)('ios-deploy', iosDeployInstallArgs, {
                encoding: 'utf8',
            });
            if (iosDeployOutput.error) {
                kaxDeployTask.fail(`Installation failed. Make sure you have run 'npm install -g ios-deploy'.`);
            }
            else {
                kaxDeployTask.succeed();
            }
        }
        catch (e) {
            kaxDeployTask.fail(e.message);
            throw e;
        }
    }
    finally {
        ern_core_1.shell.popd();
    }
}
exports.launchOnDevice = launchOnDevice;
//# sourceMappingURL=launchOnDevice.js.map