"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseJsonFromStringOrFile = void 0;
const ern_cauldron_api_1 = require("ern-cauldron-api");
const fs_extra_1 = __importDefault(require("fs-extra"));
/**
 *
 * Parse json from a string or a file and return the
 * resulting JavaScript object
 * @param s A json string or a local file path to a json file.
 * Can also be a reference to a file stored in cauldron, using
 * the cauldron:// file scheme.
 */
async function parseJsonFromStringOrFile(s) {
    let result;
    try {
        if (s.startsWith(ern_cauldron_api_1.cauldronFileUriScheme)) {
            const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
            const file = await cauldron.getFile({
                cauldronFilePath: s,
            });
            result = JSON.parse(file.toString());
        }
        else if (await fs_extra_1.default.pathExists(s)) {
            result = await fs_extra_1.default.readJson(s);
        }
        else {
            result = JSON.parse(s);
        }
    }
    catch (e) {
        throw new Error('[parseJsonFromStringOrFile] Invalid JSON or file');
    }
    return result;
}
exports.parseJsonFromStringOrFile = parseJsonFromStringOrFile;
//# sourceMappingURL=parseJsonFromStringOrFile.js.map