"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runContainerPipeline = void 0;
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_core_1 = require("ern-core");
const ern_container_transformer_1 = require("ern-container-transformer");
const ern_container_publisher_1 = require("ern-container-publisher");
const parseJsonFromStringOrFile_1 = require("./parseJsonFromStringOrFile");
/**
 * Run a given Container pipeline
 */
async function runContainerPipeline({ containerPath, containerVersion, pipeline, platform, }) {
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    let curIdx = 0;
    for (const pipelineElt of pipeline || []) {
        curIdx++;
        const pipelineEltPackage = ern_core_1.PackagePath.fromString(pipelineElt.name);
        let pipelineEltType;
        if ((0, ern_container_publisher_1.isPublisher)(pipelineEltPackage)) {
            pipelineEltType = 'publisher';
        }
        else if ((0, ern_container_transformer_1.isTransformer)(pipelineEltPackage)) {
            pipelineEltType = 'transformer';
        }
        else {
            ern_core_1.log.error(`[${curIdx}/${pipeline.length}] Skipping non transformer/publisher pipeline element ${pipelineElt.name}`);
            continue;
        }
        if (pipelineElt.disabled) {
            ern_core_1.log.info(`[${curIdx}/${pipeline.length}] Skipping ${pipelineElt.name} ${pipelineEltType} [disabled]`);
            continue;
        }
        let extra = pipelineElt.extra;
        if (extra &&
            typeof extra === 'string' &&
            extra.startsWith(ern_cauldron_api_1.cauldronFileUriScheme)) {
            if (!(await cauldron.hasFile({ cauldronFilePath: extra }))) {
                throw new Error(`Cannot find extra config file ${extra} in Cauldron`);
            }
            const extraFile = await cauldron.getFile({ cauldronFilePath: extra });
            extra = (0, parseJsonFromStringOrFile_1.parseJsonFromStringOrFile)(extraFile.toString());
        }
        await ern_core_1.kax
            .task(`[${curIdx}/${pipeline.length}] Running Container ${pipelineEltType} ${pipelineEltPackage.basePath}`)
            .run((0, ern_container_publisher_1.isPublisher)(pipelineEltPackage)
            ? (0, ern_container_publisher_1.publishContainer)({
                containerPath,
                containerVersion,
                extra,
                inPlace: pipelineElt.inPlace,
                platform,
                publisher: ern_core_1.PackagePath.fromString(pipelineElt.name),
                url: pipelineElt.url,
            })
            : (0, ern_container_transformer_1.transformContainer)({
                containerPath,
                extra,
                platform,
                transformer: ern_core_1.PackagePath.fromString(pipelineElt.name),
            }));
    }
}
exports.runContainerPipeline = runContainerPipeline;
//# sourceMappingURL=runContainerPipeline.js.map