"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runContainerPipelineForDescriptor = void 0;
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_core_1 = require("ern-core");
const runContainerPipeline_1 = require("./runContainerPipeline");
/**
 * Given a full native application descriptor and a local Container path,
 * run the Container pipeline configured for this descriptor
 */
async function runContainerPipelineForDescriptor({ descriptor, containerPath, containerVersion, }) {
    if (!descriptor.platform || !descriptor.version) {
        throw new Error('Can only work with a full native application descriptor');
    }
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    const containerGenConfig = await ern_core_1.kax
        .task('Getting pipeline configuration from Cauldron')
        .run(cauldron.getContainerGeneratorConfig(descriptor));
    const pipeline = containerGenConfig === null || containerGenConfig === void 0 ? void 0 : containerGenConfig.pipeline;
    if (!pipeline) {
        ern_core_1.log.warn(`No pipeline configuration found for ${descriptor}`);
    }
    else {
        return (0, runContainerPipeline_1.runContainerPipeline)({
            containerPath,
            containerVersion,
            pipeline,
            platform: descriptor.platform,
        });
    }
}
exports.runContainerPipelineForDescriptor = runContainerPipelineForDescriptor;
//# sourceMappingURL=runContainerPipelineForDescriptor.js.map