"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runMiniApp = void 0;
const ern_core_1 = require("ern-core");
const ern_container_publisher_1 = require("ern-container-publisher");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const getRunnerGeneratorForPlatform_1 = require("./getRunnerGeneratorForPlatform");
const generateContainerForRunner_1 = require("./generateContainerForRunner");
const launchRunner_1 = require("./launchRunner");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
async function runMiniApp(platform, { baseComposite, cwd, descriptor, dev, extra, host, jsApiImpls, launchArgs, launchEnvVars, launchFlags, mainMiniAppName, manifestId, miniapps, platformVersion = ern_core_1.Platform.currentVersion, port, } = {}) {
    var _a, _b, _c;
    cwd = cwd !== null && cwd !== void 0 ? cwd : process.cwd();
    let napDescriptor;
    if (miniapps && !ern_core_1.MiniApp.existInPath(cwd) && !mainMiniAppName) {
        throw new Error('If you run multiple MiniApps you need to provide the name of the MiniApp to launch');
    }
    let jsMainModuleName;
    if (ern_core_1.MiniApp.existInPath(cwd)) {
        jsMainModuleName = (await fs_extra_1.default.pathExists(path_1.default.join(cwd, `index.${platform}.js`)))
            ? `index.${platform}`
            : 'index';
    }
    if (miniapps && dev) {
        dev = false;
        ern_core_1.log.warn('Turning off dev mode since you are running multiple MiniApps. \nIf you want to start a packager, execute `ern start` command with all the miniapps in a separate terminal.\nCheck this link for more details: https://native.electrode.io/cli-commands/start');
    }
    if (jsApiImpls && jsApiImpls.length > 0 && descriptor) {
        throw new Error('You cannot pass JavaScript API implementations when using a Native Application Descriptor');
    }
    if (miniapps && descriptor) {
        throw new Error('You cannot use miniapps and descriptor at the same time');
    }
    let cauldron;
    if (descriptor) {
        cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        napDescriptor = ern_core_1.utils.coerceToAppVersionDescriptor(descriptor);
    }
    const compositeGenConfig = cauldron && (await cauldron.getCompositeGeneratorConfig(napDescriptor));
    baseComposite =
        baseComposite ||
            ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
    let entryMiniAppName = mainMiniAppName || '';
    if (miniapps) {
        if (ern_core_1.MiniApp.existInPath(cwd)) {
            const miniapp = ern_core_1.MiniApp.fromPath(cwd);
            miniapps = miniapps.concat(ern_core_1.PackagePath.fromString(`file:${cwd}`));
            ern_core_1.log.debug(`This command is being run from the ${miniapp.name} MiniApp directory.`);
            ern_core_1.log.info(`All extra MiniApps will be included in the Runner container along with ${miniapp.name}`);
            if (!mainMiniAppName) {
                ern_core_1.log.info(`${miniapp.name} will be set as the main MiniApp`);
                ern_core_1.log.info(`You can select another one instead through '--mainMiniAppName' option`);
                entryMiniAppName = miniapp.name;
            }
        }
    }
    else if (!miniapps && !descriptor) {
        entryMiniAppName = ern_core_1.MiniApp.fromCurrentPath().name;
        miniapps = [ern_core_1.PackagePath.fromString(`file:${cwd}`)];
        ern_core_1.log.debug(`This command is being run from the ${entryMiniAppName} MiniApp directory.`);
        ern_core_1.log.debug(`Initializing Runner`);
        if (dev) {
            await ern_core_1.reactnative.startPackagerInNewWindow({
                cwd,
                host,
                port,
            });
        }
        else {
            ern_core_1.log.info('Dev mode not enabled, will not start the packager.');
        }
    }
    else {
        miniapps =
            (cauldron &&
                napDescriptor &&
                (await cauldron.getContainerMiniApps(napDescriptor))) ||
                [];
    }
    if (descriptor) {
        jsApiImpls =
            (cauldron &&
                napDescriptor &&
                (await cauldron.getContainerJsApiImpls(napDescriptor))) ||
                [];
    }
    if ((_a = extra === null || extra === void 0 ? void 0 : extra.compositeGenerator) === null || _a === void 0 ? void 0 : _a.metroExtraNodeModules) {
        Object.keys(extra.compositeGenerator.metroExtraNodeModules).map((value) => {
            const moduleValue = extra.compositeGenerator.metroExtraNodeModules[value];
            extra.compositeGenerator.metroExtraNodeModules[value] = path_1.default.isAbsolute(moduleValue)
                ? moduleValue
                : path_1.default.join(cwd, 'node_modules', moduleValue);
        });
    }
    const outDir = ern_core_1.Platform.getContainerGenOutDirectory(platform);
    const containerGenResult = await (0, generateContainerForRunner_1.generateContainerForRunner)(platform, {
        baseComposite,
        extra,
        jsApiImpls,
        jsMainModuleName,
        miniApps: miniapps,
        napDescriptor: napDescriptor || undefined,
        outDir,
    });
    const pathToRunner = path_1.default.join(cwd, platform);
    const oldAndroidRunner = isOldAndroidRunner();
    if (platform === 'android') {
        if (oldAndroidRunner) {
            ern_core_1.log.warn('Looks like you are running on an older version of the android runner project generated using ern version < 0.38.0. \n We recommend that you delete your current Android directory and regenerate the new runner by executing `ern run-android` command again.');
        }
        extra = extra || {};
        extra.androidConfig = {
            ...extra === null || extra === void 0 ? void 0 : extra.androidConfig,
            artifactId: oldAndroidRunner
                ? `runner-ern-container`
                : `runner-ern-container-${entryMiniAppName.toLowerCase()}`,
            groupId: 'com.walmartlabs.ern',
            packageFilePath: oldAndroidRunner
                ? 'com/walmartlabs/ern'
                : `com/walmartlabs/ern/${entryMiniAppName.toLowerCase()}`,
            packageName: oldAndroidRunner
                ? `com.walmartlabs.ern`
                : `com.walmartlabs.ern.${entryMiniAppName.toLowerCase()}`,
        };
        const mavenPublisher = (await ern_core_1.kax
            .task('Querying Manifest for ern-container-publisher-maven version to use')
            .run(ern_core_1.manifest.getNativeDependency(ern_core_1.PackagePath.fromString('ern-container-publisher-maven'), {
            manifestId,
            platformVersion,
        }))) || ern_core_1.PackagePath.fromString('ern-container-publisher-maven');
        await (0, ern_container_publisher_1.publishContainer)({
            containerPath: outDir,
            containerVersion: '1.0.0',
            extra: extra.androidConfig,
            platform: 'android',
            publisher: mavenPublisher,
            url: (0, ern_core_1.getDefaultMavenLocalDirectory)(),
        });
    }
    const compositeNativeDeps = await containerGenResult.config.composite.getNativeDependencies({});
    const reactNativeDep = lodash_1.default.find(compositeNativeDeps.all, (p) => p.name === 'react-native');
    const runnerGeneratorConfig = {
        extra: {
            androidConfig: (_b = extra === null || extra === void 0 ? void 0 : extra.androidConfig) !== null && _b !== void 0 ? _b : {},
            containerGenWorkingDir: ern_core_1.Platform.containerGenDirectory,
            iosConfig: (_c = extra === null || extra === void 0 ? void 0 : extra.iosConfig) !== null && _c !== void 0 ? _c : {},
        },
        mainMiniAppName: entryMiniAppName,
        outDir: pathToRunner,
        reactNativeDevSupportEnabled: dev,
        reactNativePackagerHost: host,
        reactNativePackagerPort: port,
        reactNativeVersion: reactNativeDep.version,
        targetPlatform: platform,
    };
    if (!(await fs_extra_1.default.pathExists(pathToRunner))) {
        ern_core_1.shell.mkdir('-p', pathToRunner);
        await ern_core_1.kax
            .task(`Generating ${platform} Runner project`)
            .run((0, getRunnerGeneratorForPlatform_1.getRunnerGeneratorForPlatform)(platform).generate(runnerGeneratorConfig));
    }
    else {
        await ern_core_1.kax
            .task(`Regenerating ${platform} Runner Configuration`)
            .run((0, getRunnerGeneratorForPlatform_1.getRunnerGeneratorForPlatform)(platform).regenerateRunnerConfig(runnerGeneratorConfig));
    }
    function isOldAndroidRunner() {
        const paths = path_1.default.join(pathToRunner, 'app/src/main/java/com/walmartlabs/ern/RunnerConfig.java');
        return fs_extra_1.default.pathExistsSync(paths);
    }
    const launchRunnerConfig = {
        extra: {
            launchArgs,
            launchEnvVars,
            launchFlags,
        },
        pathToRunner,
        platform,
    };
    if (platform === 'android') {
        launchRunnerConfig.extra.packageName = extra.androidConfig.packageName;
    }
    await (0, launchRunner_1.launchRunner)(launchRunnerConfig);
}
exports.runMiniApp = runMiniApp;
//# sourceMappingURL=runMiniApp.js.map