"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.syncCauldronContainer = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const container_1 = require("./container");
const runContainerPipelineForDescriptor_1 = require("./runContainerPipelineForDescriptor");
const constants = __importStar(require("./constants"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const composite_1 = require("./composite");
const fs_extra_1 = __importDefault(require("fs-extra"));
async function syncCauldronContainer(stateUpdateFunc, descriptor, commitMessage, { containerVersion, resetCache, sourceMapOutput, } = {}) {
    if (!descriptor.platform) {
        throw new Error(`${descriptor} does not specify a platform`);
    }
    const platform = descriptor.platform;
    const outDir = ern_core_1.Platform.getContainerGenOutDirectory(platform);
    let cauldronContainerNewVersion;
    let cauldron;
    try {
        cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        // ================================================================
        // Set new Container version
        // ================================================================
        if (containerVersion) {
            cauldronContainerNewVersion = containerVersion;
        }
        else {
            const detachContainerVersionFromRoot = await cauldron.getConfigForKey('detachContainerVersionFromRoot', descriptor);
            cauldronContainerNewVersion = detachContainerVersionFromRoot
                ? await cauldron.getContainerVersion(descriptor)
                : await cauldron.getTopLevelContainerVersion(descriptor);
            if (cauldronContainerNewVersion) {
                if (!semver_1.default.valid(cauldronContainerNewVersion)) {
                    throw new Error(`${cauldronContainerNewVersion} is not a semver compliant version and therefore cannot be auto patch incremented.
Please set the new container version through command options.`);
                }
                cauldronContainerNewVersion = semver_1.default.inc(cauldronContainerNewVersion, 'patch');
            }
            else {
                // Default to 1.0.0 for Container version
                cauldronContainerNewVersion = '1.0.0';
            }
        }
        // Begin a Cauldron transaction
        await cauldron.beginTransaction();
        // Trigger state change in Cauldron
        await stateUpdateFunc();
        // ================================================================
        // Generate Composite from Cauldron
        // ================================================================
        const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(descriptor);
        const baseComposite = (compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
            ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite);
        const compositeDir = (0, ern_core_1.createTmpDir)();
        const composite = await ern_core_1.kax.task('Generating Composite from Cauldron').run((0, composite_1.runCauldronCompositeGen)(descriptor, {
            baseComposite,
            outDir: compositeDir,
        }));
        // Sync native dependencies in Cauldron
        const compositeNativeDeps = await composite.getInjectableNativeDependencies(platform);
        await cauldron.setNativeDependenciesInContainer(descriptor, compositeNativeDeps);
        // Generate Container from Cauldron
        sourceMapOutput = sourceMapOutput !== null && sourceMapOutput !== void 0 ? sourceMapOutput : path_1.default.join((0, ern_core_1.createTmpDir)(), 'index.map');
        const containerGenRes = await ern_core_1.kax
            .task('Generating Container from Cauldron')
            .run((0, container_1.runCauldronContainerGen)(descriptor, composite, {
            outDir,
            resetCache,
            sourceMapOutput,
        }));
        // Update container version in Cauldron
        await cauldron.updateContainerVersion(descriptor, cauldronContainerNewVersion);
        // Update version of ern used to generate this Container
        await cauldron.updateContainerErnVersion(descriptor, ern_core_1.Platform.currentVersion);
        // Update yarn lock and run Container transformers sequentially
        const pathToNewYarnLock = path_1.default.join(compositeDir, 'yarn.lock');
        await cauldron.addOrUpdateYarnLock(descriptor, constants.CONTAINER_YARN_KEY, pathToNewYarnLock);
        // Run container pipeline (transformers/publishers)
        await ern_core_1.kax.task('Running Container Pipeline').run((0, runContainerPipelineForDescriptor_1.runContainerPipelineForDescriptor)({
            containerPath: outDir,
            containerVersion: cauldronContainerNewVersion,
            descriptor,
        }));
        // Upload source map if a source map store server is configured
        const sourcemapStoreConfig = await cauldron.getSourceMapStoreConfig();
        if (sourcemapStoreConfig) {
            try {
                const sdk = new ern_core_1.SourceMapStoreSdk(sourcemapStoreConfig.url);
                await ern_core_1.kax
                    .task(`Uploading source map to source map store [${sourcemapStoreConfig.url}]`)
                    .run(sdk.uploadContainerSourceMap({
                    containerVersion: cauldronContainerNewVersion,
                    descriptor,
                    sourceMapPath: sourceMapOutput,
                }));
            }
            catch (e) {
                ern_core_1.log.error(`Source map upload failed : ${e}`);
            }
        }
        // Upload source map to bugsnag if configured
        const bugsnagConfig = await cauldron.getBugsnagConfig(descriptor);
        if (bugsnagConfig) {
            try {
                const { apiKey } = bugsnagConfig;
                const [minifiedFile, minifiedUrl, projectRoot, sourceMap] = [
                    await fs_extra_1.default.realpath(containerGenRes.bundlingResult.bundlePath),
                    path_1.default.basename(containerGenRes.bundlingResult.bundlePath),
                    await fs_extra_1.default.realpath(path_1.default.join(composite.path, 'node_modules')),
                    await fs_extra_1.default.realpath(containerGenRes.bundlingResult.sourceMapPath),
                ];
                const compositeMiniApps = await composite.getMiniAppsPackages();
                await (0, ern_core_1.bugsnagUpload)({
                    apiKey,
                    minifiedFile,
                    minifiedUrl,
                    projectRoot,
                    sourceMap,
                    uploadNodeModules: false,
                    uploadSources: false,
                });
            }
            catch (e) {
                ern_core_1.log.error(`Bugsnag upload failed : ${e}`);
            }
        }
        // Commit Cauldron transaction
        await ern_core_1.kax
            .task('Updating Cauldron')
            .run(cauldron.commitTransaction(commitMessage));
        ern_core_1.log.info(`Added new container version ${cauldronContainerNewVersion} for ${descriptor} in Cauldron`);
    }
    catch (e) {
        ern_core_1.log.error(`[syncCauldronContainer] An error occurred: ${e}`);
        if (cauldron) {
            cauldron.discardTransaction();
        }
        throw e;
    }
}
exports.syncCauldronContainer = syncCauldronContainer;
//# sourceMappingURL=syncCauldronContainer.js.map